/*-------------------------------------------------------------------------+
| Copyright 2011 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart.figure;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Shape;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Path;

/**
 * This shape consists of an ellipse that is surrounded by transparent insets.
 * It is filled with {@link #getBackgroundColor()}. A light top-down gradient
 * from white to the background color is painted over the background color to
 * achieve a 3 dimensional reflection effect.
 * <p>
 * Anti-aliasing is enabled.
 * 
 * @author hummel
 * @author hoelzl
 */
public class PrettyInsetEllipse extends Shape {

	/** The transparent insets surrounding this shape. */
	protected final Insets insets;

	/** Constructor. */
	public PrettyInsetEllipse(Insets insets) {
		this.insets = insets;
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Paints a rounded rectangle with a top-down gradient from white to background color.
	 */
	@Override
	protected void fillShape(Graphics gfx) {
		gfx.pushState();
		Rectangle croppedBounds = getCroppedBounds();

		// Traditional style is default.
		gfx.fillOval(croppedBounds);
		gfx.setForegroundColor(ColorConstants.white);
		gfx.setAlpha(200);

		Path path = new Path(null);
		path.addArc(croppedBounds.x, croppedBounds.y, croppedBounds.width, croppedBounds.height, 0,
				360);
		gfx.setClip(path);
		gfx.fillGradient(croppedBounds, true);
		gfx.popState();
		path.dispose();
	}

	/** Returns a copy of the bounds cropped by the insets. */
	protected Rectangle getCroppedBounds() {
		return bounds.getShrinked(insets);
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Additionally enables anti-aliasing.
	 */
	@Override
	protected void outlineShape(Graphics gfx) {
		gfx.pushState();
		gfx.setAntialias(SWT.ON);
		gfx.drawOval(getCroppedBounds().resize(-1, -1));
		gfx.popState();
	}
}
