/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.annotation.valueprovider;

import org.fortiss.tooling.base.annotation.AnnotationEntry;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;

/**
 * Container class to uniquely specify an {@link IAnnotatedSpecification}, linking it with its
 * {@link AnnotationEntry}.
 * 
 * @author diewald
 */
public class AnnotationInstSpec<T extends IAnnotatedSpecification> {
	/**
	 * {@link AnnotationEntry} from which a particular {@link IAnnotatedSpecification} is to be
	 * displayed in the column represented by this {@link AnnotationInstSpec}.
	 */
	private AnnotationEntry entry;

	/**
	 * {@link IAnnotatedSpecification} from {@code entry} to be displayed in the column
	 * represented by this {@link AnnotationInstSpec}.
	 */
	private IAnnotatedSpecification annotatedSpecification;

	/** Constructor. */
	public AnnotationInstSpec(AnnotationEntry entry, T annotatedSpecification) {
		this.entry = entry;
		this.annotatedSpecification = annotatedSpecification;
	}

	/**
	 * Returns the {@link AnnotationEntry} from which a particular {@link IAnnotatedSpecification}
	 * is to be displayed in the column represented by this {@link AnnotationInstSpec}.
	 */
	public AnnotationEntry getEntry() {
		return entry;
	}

	/**
	 * Returns the {@link IAnnotatedSpecification} from {@code entry} to be displayed in the
	 * column represented by this {@link AnnotationInstSpec}.
	 */
	@SuppressWarnings("unchecked")
	public Class<T> getAnnotatedSpecificationType() {
		return (Class<T>)annotatedSpecification.getClass();
	}

	/**
	 * Returns the {@link IAnnotatedSpecification} from {@code entry} to be displayed in the
	 * column represented by this {@link AnnotationInstSpec}.
	 */
	public IAnnotatedSpecification getAnnotatedSpecification() {
		return annotatedSpecification;
	}

	/**
	 * <p>
	 * The hashcode of the {@link AnnotationInstSpec} is ONLY based on the annotation type to
	 * represent one particular annotation instance type. Nevertheless, the
	 * {@link AnnotationInstSpec} also carries references to the {@link AnnotationEntry} and the
	 * {@link IAnnotatedSpecification} instances to allow a backtracking to the originating
	 * elements.
	 * 
	 * {@inheritDoc}
	 */
	@Override
	public int hashCode() {
		return getAnnotatedSpecificationType().hashCode();
	}

	/** {@inheritDoc} */
	@Override
	public boolean equals(Object other) {
		if(other instanceof AnnotationInstSpec) {
			return this.hashCode() == other.hashCode();
		}
		return super.equals(other);
	}
}
