/*-------------------------------------------------------------------------+
| Copyright 2011 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.service;

import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.xml.type.AnyType;
import org.fortiss.tooling.kernel.extension.IMigrationProvider;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.internal.MigrationService;

/**
 * The migration service checks and upgrades the old models from the last release.
 * 
 * @author mou
 */
public interface IMigrationService {
	/** Returns the service instance. */
	public static IMigrationService getInstance() {
		return MigrationService.getInstance();
	}

	/** Checks whether the migration is needed. */
	boolean needMigration(ITopLevelElement input, Map<EObject, AnyType> unknownFeatures);

	/** Performs the migration for the given input. */
	void migrate(ITopLevelElement input, Map<EObject, AnyType> unknownFeatures);

	/** Registers the migration provider with the service. */
	void registerMigrationProvider(IMigrationProvider provider, Class<?> modelElementClass);
}
